package envoy.event;

import envoy.data.Contact;
import envoy.data.Group;
import envoy.data.User;

/**
 * Project: <strong>envoy-common</strong><br>
 * File: <strong>GroupResizeEvent.java</strong><br>
 * Created: <strong>25 Mar 2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy Common v0.1-beta
 */
public class GroupResizeEvent extends Event<Long> {

	private final long		groupID;
	private OperationType	operationType;

	private static final long serialVersionUID = 0L;

	/**
	 * Project: <strong>envoy-common</strong><br>
	 * File: <strong>GroupResizeEvent.java</strong><br>
	 * Created: <strong>25 Mar 2020</strong><br>
	 *
	 * @author Leon Hofmeister
	 * @since Envoy Common v0.1-beta
	 */
	public enum OperationType {
		/**
		 *
		 */
		ADD,
		/**
		 *
		 */
		REMOVE
	}

	/**
	 * Initializes a {@link GroupResizeEvent} through a Contact where the name has
	 * already been set.
	 *
	 * @param user          the {@link User} who wants to join or leave a group
	 * @param group         the {@link Group} he wants to join or leave
	 * @param operationType whether the user should be removed from this group. If
	 *                      false,
	 *                      this user will be added
	 * @since Envoy Common v0.2-alpha
	 */
	public GroupResizeEvent(User user, Group group, OperationType operationType) {
		super(user.getID());
		if (group.getMemberIDs().contains(user.getID())) {
			if (operationType.equals(OperationType.ADD)) throw new IllegalStateException(
					"Cannot add " + user + " to group " + group.getID() + " because he is already a member of this group");
		} else if (operationType.equals(OperationType.REMOVE))
			throw new IllegalStateException("Cannot remove " + user + " from group " + group.getID() + " because he is no part of this group");
		groupID				= group.getID();
		this.operationType	= operationType;
	}

	/**
	 * @return the ID of the {@link Contact} this event is related to
	 * @since Envoy Common v0.2-alpha
	 */
	public long getGroupId() { return groupID; }

	/**
	 * @return the operationType
	 * @since Envoy Common v0.1-beta
	 */
	public OperationType getOperationType() { return operationType; }

	/**
	 * @param operationType the operationType to set
	 * @since Envoy Common v0.1-beta
	 */
	public void setOperationType(OperationType operationType) { this.operationType = operationType; }

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() { return String.format("GroupResizeEvent[userid=%d,groupid=%d,operationType=%b]", get(), groupID, operationType); }
}